package jp.gihyo.rest.clock;

import static org.hamcrest.core.Is.is;
import static org.hamcrest.core.IsNot.not;
import static org.hamcrest.core.IsNull.nullValue;
import static org.junit.Assert.assertThat;
import static org.junit.matchers.StringContains.containsString;

import java.util.Calendar;
import java.util.Date;

import jp.gihyo.rest.clock.util.ClockWrapper;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.restlet.Application;
import org.restlet.Component;
import org.restlet.data.Protocol;

import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.PostMethodWebRequest;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebResponse;


public class WorldTimeClockTest {
    private Component component;
    private WebConversation wc;

    @Before
    public void コンポーネント準備() throws Exception {
        component = new Component();
        component.getServers().add(Protocol.HTTP, 8182);
        Application app = new ClockApplication(component.getContext());
        component.getDefaultHost().attach("/time", app);
        component.start();

        wc = new WebConversation();
        wc.setExceptionsThrownOnErrorStatus(false);
    }

    @After
    public void コンポーネント終了処理() throws Exception {
        ClockWrapper.clear();
        component.stop();
    }

    public Date november(int day, int year) {
        Calendar c = Calendar.getInstance();
        c.clear();
        c.set(year, Calendar.NOVEMBER, day);
        return c.getTime();
    }

    @Test
    public void Tokyo時間をGETする() throws Exception {
        // テストを成功させるためにクロックラッパーに時間を設定する
        ClockWrapper.setFakeDate(november(20, 2005));

        GetMethodWebRequest req =
            new GetMethodWebRequest("http://localhost:8182/time/Asia/Tokyo");
        req.setHeaderField("Accept", "text/plain");
        WebResponse resp = wc.getResource(req);

        assertThat(resp.getResponseCode(), is(200));
        assertThat(resp.getText(), is("2005-11-20T00:00:00+0900"));
    }

    @Test
    public void GMTをGETする() throws Exception {
        ClockWrapper.setFakeDate(november(20, 2005));

        GetMethodWebRequest req =
            new GetMethodWebRequest("http://localhost:8182/time/GMT");
        req.setHeaderField("Accept", "text/plain");
        WebResponse resp = wc.getResource(req);

        assertThat(resp.getResponseCode(), is(200));
        assertThat(resp.getText(), is("2005-11-19T15:00:00+0000"));
    }

    @Test
    public void コンテントネゴシエーション可能であること() throws Exception {
        ClockWrapper.setFakeDate(november(20, 2005));
        GetMethodWebRequest req =
            new GetMethodWebRequest("http://localhost:8182/time/Asia/Tokyo");
        req.setHeaderField("Accept", "application/xhtml+xml");
        WebResponse resp = wc.getResource(req);
        assertThat(resp.getContentType(), is("application/xhtml+xml"));
    }

    @Test
    public void 存在しないタイムゾーンをGETすると404が返る() throws Exception {
        GetMethodWebRequest req =
            new GetMethodWebRequest("http://localhost:8182/time/foo/bar");
        WebResponse resp = wc.getResource(req);
        assertThat(resp.getResponseCode(), is(404));
    }

    @Test
    public void ルートリソースをXHTMLで返す() throws Exception {
        GetMethodWebRequest req =
            new GetMethodWebRequest("http://localhost:8182/time");
        WebResponse resp = wc.getResource(req);

        assertThat(resp.getResponseCode(), is(200));
        assertThat(resp.getContentType(), is("application/xhtml+xml"));
        String link = "<a href=\"http://localhost:8182/time/Asia/Tokyo\">";
        assertThat(resp.getText(), containsString(link));
    }

    @Test
    public void リストURIに対してカスタムタイムゾーンをPOSTする() throws Exception {
        PostMethodWebRequest req =
            new PostMethodWebRequest("http://localhost:8182/time");
        req.setParameter("customId", "GMT+10");
        WebResponse resp = wc.getResource(req);

        assertThat(resp.getResponseCode(), is(201));
        String locationHeader = resp.getHeaderField("location");
        assertThat(locationHeader, is(not(nullValue())));
        assertThat("作成されたURIをlocationヘッダで返す", locationHeader,
                is("http://localhost:8182/time/GMT+10"));
    }

    @Test
    public void リストURIに対して既存のTimeZoneIdをPOSTすると409が返る() throws Exception {
        PostMethodWebRequest req =
            new PostMethodWebRequest("http://localhost:8182/time");
        req.setParameter("customId", "Asia/Tokyo");
        WebResponse resp = wc.getResource(req);

        assertThat(resp.getResponseCode(), is(409));
        String locationHeader = resp.getHeaderField("location");
        assertThat(locationHeader, is(not(nullValue())));
        assertThat("存在するURIをlocationヘッダで返す", locationHeader,
                is("http://localhost:8182/time/Asia/Tokyo"));
    }

    @Test
    public void リストURIに対して不正なTimeZoneIdをPOSTすると400が返る() throws Exception {
        PostMethodWebRequest req =
            new PostMethodWebRequest("http://localhost:8182/time");
        req.setParameter("customId", "hoge");
        WebResponse resp = wc.getResource(req);

        assertThat(resp.getResponseCode(), is(400));
    }
}
